/**
 * 
 */
package cmis.junit.examples;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.UUID;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.oasis_open.docs.ns.cmis.core._200901.CmisAllowableActionsType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisChangeEventType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisObjectType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertiesType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisProperty;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyBoolean;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyDateTime;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyId;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyInteger;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyString;
import org.oasis_open.docs.ns.cmis.core._200901.CmisPropertyUri;
import org.oasis_open.docs.ns.cmis.core._200901.CmisQueryType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisRepositoryCapabilitiesType;
import org.oasis_open.docs.ns.cmis.core._200901.CmisRepositoryInfoType;
import org.oasis_open.docs.ns.cmis.core._200901.EnumBaseObjectType;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityChanges;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityJoin;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCapabilityQuery;
import org.oasis_open.docs.ns.cmis.core._200901.EnumCollectionType;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesDocument;
import org.oasis_open.docs.ns.cmis.core._200901.EnumPropertiesFolder;
import org.oasis_open.docs.ns.cmis.core._200901.EnumTypeOfChanges;
import org.oasis_open.docs.ns.cmis.restatom._200901.CmisAtomChildren;
import org.w3._2005.atom.AtomDateConstruct;
import org.w3._2005.atom.AtomPersonConstruct;
import org.w3._2005.atom.AtomTextConstruct;
import org.w3._2005.atom.Content;
import org.w3._2005.atom.EntryType;
import org.w3._2005.atom.FeedType;
import org.w3._2005.atom.Id;
import org.w3._2005.atom.Link;
import org.w3._2007.app.AppCollectionType;
import org.w3._2007.app.AppServiceType;
import org.w3._2007.app.AppWorkspaceType;

/**
 * @author abrown
 * 
 */
public class ExampleTest extends TestCase {

	private static final String _CORE = "org.oasis_open.docs.ns.cmis.core._200901";
	private static final String _MSG = "org.oasis_open.docs.ns.cmis.messaging._200901";
	private static final String _RA = "org.oasis_open.docs.ns.cmis.restatom._200901";
	private static final String _ATOM = "org.w3._2005.atom";
	private static final String _APP = "org.w3._2007.app";

	private static final String _CMIS_VERSION = "0.61";

	private static final String AUTHOR_NAME = "Al Brown";
	private static final String AUTHOR_EMAIL = "albertcbrown@us.ibm.com";
	private static final String AUTHOR_URI = "http://www.ibm.com/";

	private static final String CS_LENGTH = "4123";
	private static final String CS_MIMETYPE = "text/plain";

	private static final String URL_BASE = "http://cmisexample.oasis-open.org/rep1/";

	public static final String MIMETYPE_QUERY = "application/cmisquery+xml";
	public static final String MIMETYPE_ATOM = "application/atom+xml";
	public static final String MIMETYPE_ATOMENTRY = "application/atom+xml;type=entry";
	public static final String MIMETYPE_ALLOWABLEACTIONS = "application/cmis+xml;type=allowableActions";
	public static final String MIMETYPE_ATOMFEED = "application/atom+xml;type=feed";
	public static final String MIMETYPE_TYPE = "application/cmis+xml;type=type";
	public static final String MIMETYPE_APPSERVICE = "application/atomsvc+xml";
	public static final String MIMETYPE_TEXT = "text/plain";
	public static final String MIMETYPE_HTML = "text/html";
	private static String _PRODUCT_NAME = "CMIS Prototype for VendorX";
	private static String _PRODUCT_VERSION = _CMIS_VERSION;

	private org.oasis_open.docs.ns.cmis.core._200901.ObjectFactory coreFactory = new org.oasis_open.docs.ns.cmis.core._200901.ObjectFactory();
	private org.oasis_open.docs.ns.cmis.restatom._200901.ObjectFactory raFactory = new org.oasis_open.docs.ns.cmis.restatom._200901.ObjectFactory();
	private org.w3._2005.atom.ObjectFactory atomFactory = new org.w3._2005.atom.ObjectFactory();
	private org.w3._2007.app.ObjectFactory appFactory = new org.w3._2007.app.ObjectFactory();

	private static final String JAXB_PACKAGE_LIST = _CORE + ":" + _MSG + ":"
			+ _ATOM + ":" + _APP + ":" + _RA;

	private Marshaller marshaller;
	private JAXBContext jaxbContext;

	public void test01AllowableActions() {
		CmisAllowableActionsType aa = getAllowableActions(EnumBaseObjectType.DOCUMENT);
		aa.setParentId("object_id_1");
		aa.setParentUrl("http://foo/objectid1/allowableactions");

		// write it
		JAXBElement<CmisAllowableActionsType> element = coreFactory
				.createAllowableActions(aa);
		writeOutput("AllowableActions", element);
	}

	public EntryType getAtomEntryType(String objectId,
			EnumBaseObjectType bot, String typeid, boolean checkedout,
			boolean allowableActions, boolean changeEntry, String name) {
		EntryType e = new EntryType();

		// <atom:author xml:base="http://tempuri.org" xml:lang="">
		// <atom:name>Al Brown</atom:name>
		// </atom:author>
		e.getAuthor().add(getAtomPerson(AUTHOR_NAME, AUTHOR_EMAIL, AUTHOR_URI));

		// <!-- atom:content -->
		// <atom:content src="http://www.cmis.org/rep1/media/document-entry">
		// </atom:content>

		Content c = new Content();
		c.setSrc(URL_BASE + objectId);
		e.setContent(c);

		// <!-- IRI to document entry resource -->
		// <atom:id>http://www.cmis.org/rep1/document-entry</atom:id>
		Id id = new Id();
		id.setContent("urn:uuid:" + objectId);
		e.setId(id);

		// <!-- atom links -->
		// <atom:link rel='self' href="http://www.cmis.org/rep1/document-entry"
		// />
		e.getLink()
				.add(
						getAtomLink("self", URL_BASE + objectId, null, null,
								null, null));

		// <atom:link rel="edit" href="http://www.cmis.org/rep1/document-entry"
		// />
		e.getLink()
				.add(
						getAtomLink("edit", URL_BASE + objectId, null, null,
								null, null));

		// <atom:link rel="cmis-allowableactions"
		// href="http://www.cmis.org/rep1/document-entry/actions" />
		e.getLink().add(
				getAtomLink("allowableactions", URL_BASE + objectId
						+ "/allowableactions", MIMETYPE_ALLOWABLEACTIONS, null,
						null, null));

		// <atom:link rel="cmis-type" href="http://www.cmis.org/rep1/type/email"
		// />
		e.getLink().add(
				getAtomLink("type", URL_BASE + objectId + "/type",
						MIMETYPE_ATOMENTRY, null, null, null));

		// <!-- createdDate -->
		// <atom:published>2001-12-31T12:00:00</atom:published>
		e.setPublished(getAtomDateConstruct(new Date()));

		// <atom:summary>
		// <!-- auto-generated HTML table of properties -->
		// </atom:summary>
		e.setSummary(getAtomText("HTML summary of Entry  " + objectId, "html"));

		// <atom:title>Document Entry example</atom:title>
		e.setTitle(getAtomText(name, "text"));

		// <!-- lastModifiedDate -->
		// <atom:updated>2001-12-31T12:00:00</atom:updated>
		e.setUpdated(getAtomDateConstruct(new Date()));

		CmisObjectType cot = new CmisObjectType();
		CmisPropertiesType cpt = new CmisPropertiesType();

		// Bools
		cpt.getProperty().add(
				getPropertyBoolean(EnumPropertiesDocument.IS_IMMUTABLE.value(),
						false));

		// Dates
		cpt.getProperty().add(
				getPropertyDate(EnumPropertiesDocument.CREATION_DATE.value(),
						new Date()));
		cpt.getProperty().add(
				getPropertyDate(EnumPropertiesDocument.LAST_MODIFICATION_DATE
						.value(), new Date()));

		// Ids
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesDocument.OBJECT_ID.value(),
						objectId));
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesDocument.OBJECT_TYPE_ID.value(),
						typeid));
		cpt.getProperty().add(
				getPropertyId(EnumPropertiesDocument.BASE_TYPE_ID.value(), bot
						.value()));

		// Integers

		// Strings
		cpt.getProperty().add(
				getPropertyString(EnumPropertiesDocument.LAST_MODIFIED_BY
						.value(), AUTHOR_NAME));
		cpt.getProperty().add(
				getPropertyString(EnumPropertiesDocument.CREATED_BY.value(),
						AUTHOR_NAME));

		// Uri

		cot.setProperties(cpt);

		if (allowableActions)
			cot.setAllowableActions(getAllowableActions(bot));

		e.getAnyOther().add(coreFactory.createObject(cot));

		switch (bot) {
		case DOCUMENT:
			cpt.getProperty().add(
					getPropertyBoolean(EnumPropertiesDocument.IS_LATEST_VERSION
							.value(), true));
			cpt
					.getProperty()
					.add(
							getPropertyBoolean(
									EnumPropertiesDocument.IS_VERSION_SERIES_CHECKED_OUT
											.value(), checkedout));
			cpt.getProperty().add(
					getPropertyBoolean(EnumPropertiesDocument.IS_MAJOR_VERSION
							.value(), false));
			cpt.getProperty().add(
					getPropertyBoolean(
							EnumPropertiesDocument.IS_LATEST_MAJOR_VERSION
									.value(), false));

			cpt.getProperty().add(
					getPropertyString(EnumPropertiesDocument.CHECKIN_COMMENT
							.value(), "Checkin comment"));
			cpt.getProperty().add(
					getPropertyString(EnumPropertiesDocument.VERSION_LABEL
							.value(), "0.1"));
			cpt.getProperty().add(
					getPropertyString(
							EnumPropertiesDocument.CONTENT_STREAM_MIME_TYPE
									.value(), "text/plain"));
			cpt.getProperty().add(
					getPropertyString(
							EnumPropertiesDocument.CONTENT_STREAM_FILENAME
									.value(), "text.txt"));
			cpt.getProperty().add(
					getPropertyUri(EnumPropertiesDocument.CONTENT_STREAM_URI
							.value(),
							"http://cmis-server.foo/document-entry/media"));
			cpt.getProperty().add(
					getPropertyInt(EnumPropertiesDocument.CONTENT_STREAM_LENGTH
							.value(), 4234));

			// provide an MVP for a doc - keywords
			CmisPropertyString cps = new CmisPropertyString();
			cps.setName("keywords");
			cps.getValue().add("document");
			cps.getValue().add("example");
			cps.getValue().add("sample");
			cps.getValue().add("cmis");
			cpt.getProperty().add(cps);

			// <atom:link rel="edit-media"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("edit-media", URL_BASE + objectId
							+ "/edit-media", null, null, null, null));

			// <atom:link rel="alternate"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("alternate",
							URL_BASE + objectId + "/alternate", null, null,
							null, null));

			// <!-- CMIS Links -->
			// <atom:link rel="cmis-parents"
			// href="http://www.cmis.org/rep1/document-entry/parents" />
			e.getLink().add(
					getAtomLink("parents", URL_BASE + objectId + "/parents",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="cmis-allversions"
			// href="http://www.cmis.org/rep1/document-entry/allversions" />
			e.getLink().add(
					getAtomLink("allversions", URL_BASE + objectId
							+ "/allversions", MIMETYPE_ATOMFEED, null, null,
							null));

			// <atom:link rel="cmis-latestversion"
			// href="http://www.cmis.org/rep1/document-entry/latestversion" />
			e.getLink().add(
					getAtomLink("latestversion", URL_BASE + objectId
							+ "/latestversions", MIMETYPE_ATOMENTRY, null,
							null, null));

			// <atom:link rel="cmis-stream"
			// href="http://www.cmis.org/rep1/media/document-entry" />
			e.getLink().add(
					getAtomLink("stream", URL_BASE + objectId + "media",
							CS_MIMETYPE, CS_LENGTH, null, null));
			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			if (checkedout) {
				cpt
						.getProperty()
						.add(
								getPropertyId(
										EnumPropertiesDocument.VERSION_SERIES_CHECKED_OUT_ID
												.value(), "vs-" + objectId));
				cpt
						.getProperty()
						.add(
								getPropertyString(
										EnumPropertiesDocument.VERSION_SERIES_CHECKED_OUT_BY
												.value(), AUTHOR_NAME));
				e.getLink().add(
						getAtomLink("pwc", URL_BASE + objectId + "/pwc",
								MIMETYPE_ATOMFEED, null, null, null));

			}

			break;
		case FOLDER:
			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			// <atom:link rel="cmis-parent"
			// href="http://www.cmis.org/rep1/folder-entry/parent" />
			e.getLink().add(
					getAtomLink("parent", URL_BASE + objectId + "/parent",
							MIMETYPE_ATOMENTRY, null, null, null));

			// <cmis:propertyId
			// cmis:name="parentId"><cmis:value>parentFolderId1<
			// /cmis:value></cmis:propertyId>
			cpt.getProperty().add(
					getPropertyId(EnumPropertiesFolder.PARENT_ID.value(),
							objectId + "parent"));

			break;
		case POLICY:
			// <!-- CMIS Links -->
			// <atom:link rel="cmis-parents"
			// href="http://www.cmis.org/rep1/document-entry/parents" />
			e.getLink().add(
					getAtomLink("parents", URL_BASE + objectId + "/parents",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="cmis-relationships"
			// href="http://www.cmis.org/rep1/document-entry/relationships" />
			e.getLink().add(
					getAtomLink("relationships", URL_BASE + objectId
							+ "/relationships", MIMETYPE_ATOMFEED, null, null,
							null));

			break;
		case RELATIONSHIP:
			// <atom:link rel="cmis-source"
			// href="http://www.cmis.org/rep1/policy-entry/source" />
			e.getLink().add(
					getAtomLink("source", URL_BASE + objectId + "/source",
							MIMETYPE_ATOMENTRY, null, null, null));

			// <atom:link rel="cmis-target"
			// href="http://www.cmis.org/rep1/policy-entry/target" />
			e.getLink().add(
					getAtomLink("target", URL_BASE + objectId + "/target",
							MIMETYPE_ATOMENTRY, null, null, null));
			break;
		}

		// Unified search
		if (changeEntry) {
			CmisChangeEventType ccet = new CmisChangeEventType();

			ccet.setChangeType(EnumTypeOfChanges.UPDATED);
			GregorianCalendar cal = new GregorianCalendar();
			cal.setTime(new Date());
			XMLGregorianCalendar xgdt = null;
			try {
				xgdt = DatatypeFactory.newInstance().newXMLGregorianCalendar(
						cal);
			} catch (DatatypeConfigurationException ex) {
				ex.printStackTrace();
			}

			ccet.setChangeTime(xgdt);
			cot.setChangeEventInfo(ccet);
		}

		return e;

	}

	public void test10DocumentEntry() {
		EntryType e = getAtomEntryType(UUID.randomUUID().toString(),
				EnumBaseObjectType.DOCUMENT, "invoice", false, true, false,
				"CMIS Example Document");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntry", element);
	}

	public void test11DocumentEntryPWC() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(),
				EnumBaseObjectType.DOCUMENT, "invoice", true, true, false,
				"CMIS Example Document for PWC");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntryPWC", element);
	}

	public void test12FolderEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(), EnumBaseObjectType.FOLDER,
				"customer", true, true, false,
				"CMIS Example Folder as Customer type");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("FolderEntry", element);
	}

	public void test13PolicyEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(), EnumBaseObjectType.POLICY,
				"policy", true, true, false, "CMIS Example Policy");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("PolicyEntry", element);
	}

	public void test14RelationshipEntry() {
		UUID u = UUID.randomUUID();

		EntryType e = getAtomEntryType(u.toString(),
				EnumBaseObjectType.RELATIONSHIP, "compounddocument", true,
				true, false,
				"CMIS Example Relationship as Compound Document type");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("RelationshipEntry", element);
	}

	public void test15FolderChildren() {
		FeedType f = getAtomFeedType("folder1feed", "Feed for folder1", true);

		// add a couple of folders
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "invoice", true, true,
						false, "CMIS Example Folder as Invoice type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.DOCUMENT, "invoice", false, true,
						false, "CMIS Example Folder as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("FolderChildren", element);
	}

	public void test15FolderDescendants() {
		FeedType f = getAtomFeedType(UUID.randomUUID().toString(),
				"Feed for folder1", false);

		// add a couple of folders
		EntryType e = null;

		// folder1
		e = getAtomEntryType(UUID.randomUUID().toString(),
				EnumBaseObjectType.FOLDER, "customer", true, true, false,
				"CMIS Example Folder as Customer type");

		// folder1 has a couple of docs
		CmisAtomChildren cat = new CmisAtomChildren();
		cat.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));
		cat.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));
		// save
		JAXBElement<CmisAtomChildren> kids = raFactory.createChildren(cat);
		e.getAnyOther().add(kids);
		f.getEntry().add(e);

		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "customer", true, true,
						false, "CMIS Example Folder as Customer type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.DOCUMENT, "invoice", false, true,
						false, "CMIS Example Doc as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("FolderChildren", element);
	}

	private static FeedType getAtomFeedType(String id, String title,
			boolean paging) {
		FeedType f = new FeedType();
		f.setTitle(getAtomText(title, "text"));
		f.getAuthor().add(getAtomPerson(AUTHOR_NAME, AUTHOR_EMAIL, AUTHOR_URI));

		// Set update to now
		f.setUpdated(getAtomDateConstruct(new Date()));

		// set the id
		Id aId = new Id();
		aId.setContent("urn:uuid:" + UUID.randomUUID().toString());
		f.setId(aId);

		// add self link
		f.getLink().add(
				getAtomLink("self", URL_BASE + id + "/3", MIMETYPE_ATOMFEED,
						null, null, null));

		// add source
		f.getLink().add(
				getAtomLink("source", URL_BASE + id, MIMETYPE_ATOMENTRY, null,
						null, null));

		if (paging) {
			// <!-- RFC 5005 Feed Paging -->
			// <atom:link rel="first"
			// href="http://www.cmis.org/rep1/folder1/children" />
			f.getLink().add(
					getAtomLink("first", URL_BASE + id + "/first",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="next"
			// href="http://www.cmis.org/rep1/folder1/children/4" />
			f.getLink().add(
					getAtomLink("next", URL_BASE + id + "/4",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="previous"
			// href="http://www.cmis.org/rep1/folder1/children/2" />
			f.getLink().add(
					getAtomLink("previous", URL_BASE + id + "/2",
							MIMETYPE_ATOMFEED, null, null, null));

			// <atom:link rel="last"
			// href="http://www.cmis.org/rep1/folder1/children/last" />
			f.getLink().add(
					getAtomLink("last", URL_BASE + id + "/last",
							MIMETYPE_ATOMFEED, null, null, null));

		}

		return f;
	}

	public void test20Query() {
		CmisQueryType q = getQueryType("SELECT * FROM Document", 50, 0, true,
				true);

		// write it
		JAXBElement<CmisQueryType> element = coreFactory.createQuery(q);
		writeOutput("Query", element);
	}

	public void test20Service() {
		AppServiceType q = getServiceType();

		// write it
		JAXBElement<AppServiceType> element = appFactory.createService(q);
		writeOutput("Service", element);
	}

	public void test30DocumentEntryWithChanges() {
		EntryType e = getAtomEntryType(UUID.randomUUID().toString(),
				EnumBaseObjectType.DOCUMENT, "loan", false, true, true,
				"CMIS Example Document - Loan");

		// write it
		JAXBElement<EntryType> element = atomFactory.createEntry(e);
		writeOutput("DocumentEntryWithChanges", element);
	}

	public void test30ChangeLog() {
		FeedType f = getAtomFeedType("folder1feed", "changelog feed", true);

		// add a couple of folders
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "customer", true, true,
						true, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "customer", true, true,
						true, "CMIS Example Folder as Customer type"));
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.FOLDER, "invoice", true, true, true,
						"CMIS Example Folder as Invoice type"));

		// add in a doc
		f.getEntry().add(
				getAtomEntryType(UUID.randomUUID().toString(),
						EnumBaseObjectType.DOCUMENT, "invoice", false, true,
						true, "CMIS Example Folder as Invoice type"));

		// write it
		JAXBElement<FeedType> element = atomFactory.createFeed(f);
		writeOutput("ChangeLog", element);
	}

	private static CmisProperty getPropertyDate(String pName, Date date) {
		CmisPropertyDateTime p = new CmisPropertyDateTime();
		p.setName(pName);

		GregorianCalendar c = new GregorianCalendar();
		c.setTime(date);

		try {
			p.getValue().add(
					DatatypeFactory.newInstance().newXMLGregorianCalendar(c));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}

		return p;
	}

	private static CmisProperty getPropertyUri(String pName, String b) {
		CmisPropertyUri p = new CmisPropertyUri();
		p.setName(pName);
		p.getValue().add(b);

		return p;
	}

	private static CmisProperty getPropertyBoolean(String pName, boolean b) {
		CmisPropertyBoolean p = new CmisPropertyBoolean();
		p.setName(pName);
		p.getValue().add(b);

		return p;
	}

	private static CmisProperty getPropertyId(String pName, String v) {
		CmisPropertyId p = new CmisPropertyId();
		p.setName(pName);
		p.getValue().add(v);

		return p;
	}

	private static CmisProperty getPropertyInt(String pName, int v) {
		CmisPropertyInteger p = new CmisPropertyInteger();
		p.setName(pName);
		p.getValue().add(BigInteger.valueOf(v));

		return p;
	}

	private static CmisProperty getPropertyString(String pName, String v) {
		CmisPropertyString p = new CmisPropertyString();
		p.setName(pName);
		p.getValue().add(v);

		return p;
	}

	/**
	 * @author abrown
	 * 
	 * @param text
	 * @param type
	 * @return
	 */
	private static AtomTextConstruct getAtomText(String text, String type) {
		AtomTextConstruct atc = new AtomTextConstruct();
		atc.getContent().add(text);
		atc.setType(type);

		return atc;
	}

	/**
	 * @author abrown
	 * 
	 * @param date
	 * @return
	 */
	private static AtomDateConstruct getAtomDateConstruct(Date d) {
		AtomDateConstruct adc = new AtomDateConstruct();

		if (d == null) {
			return adc;
		}

		GregorianCalendar c = new GregorianCalendar();

		c.setTime(d);

		try {
			adc.setValue(DatatypeFactory.newInstance().newXMLGregorianCalendar(
					c));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}

		return adc;
	}

	/**
	 * @author abrown
	 * 
	 * @return
	 */
	private AppServiceType getServiceType() {
		AppServiceType ast = new AppServiceType();
		AppWorkspaceType awt = new AppWorkspaceType();
		ast.getWorkspace().add(awt);

		awt.setTitle(getAtomText("CMIS Repository", "text"));

		awt.getCollection().add(
				getCollectionType(EnumCollectionType.CHECKEDOUT,
						"Checkedout collection", URL_BASE + "/checkedout"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.QUERY, "Query collection",
						URL_BASE + "/query"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.ROOTCHILDREN,
						"Root children collection", URL_BASE + "/rootc"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.ROOTDESCENDANTS,
						"Root descendants collection", URL_BASE + "/rootd"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.TYPESCHILDREN,
						"Types children collection", URL_BASE + "/typec"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.TYPESDESCENDANTS,
						"Types descendants collection", URL_BASE + "/typed"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.UNFILED,
						"Unfiled collection", URL_BASE + "/unfiled"));
		awt.getCollection().add(
				getCollectionType(EnumCollectionType.CHANGES,
						"changes collection", URL_BASE + "/changes"));

		CmisRepositoryInfoType crit = new CmisRepositoryInfoType();

		crit.setCmisVersionSupported(_CMIS_VERSION);
		crit.setProductName("ProductName");
		crit.setProductVersion("0.60");
		crit.setProductName(_PRODUCT_NAME);
		crit.setProductVersion(_PRODUCT_VERSION);
		crit.setRepositoryDescription("CMIS Repository Description");
		crit.setRepositoryId("repid1");
		crit.setRepositoryName("Repository1");
		crit.setRootFolderId("rootfolder");
		crit.setVendorName("CMIS Vendor 1");
		crit.setRepositoryRelationship("self");

		// save it
		awt.getAny().add(raFactory.createRepositoryInfo(crit));

		CmisRepositoryCapabilitiesType crct = new CmisRepositoryCapabilitiesType();
		crct.setCapabilityAllVersionsSearchable(true);
		crct.setCapabilityJoin(EnumCapabilityJoin.INNERANDOUTER);
		crct.setCapabilityMultifiling(true);
		crct.setCapabilityPWCSearchable(true);
		crct.setCapabilityPWCUpdateable(true);
		crct.setCapabilityQuery(EnumCapabilityQuery.BOTHCOMBINED);
		crct.setCapabilityUnfiling(true);
		crct.setCapabilityVersionSpecificFiling(true);

		// Unified Search Proposal
		crct.setCapabilityChanges(EnumCapabilityChanges.ALL);
		crct.getCapabilityChangesOnType().add(EnumBaseObjectType.DOCUMENT);
		crct.getCapabilityChangesOnType().add(EnumBaseObjectType.FOLDER);
		crct.setChangesIncomplete(Boolean.TRUE);

		// save
		crit.setCapabilities(crct);

		return ast;
	}

	private static AppCollectionType getCollectionType(
			EnumCollectionType cType, String title, String href) {
		AppCollectionType act = new AppCollectionType();

		act.getOtherAttributes().put(
				new QName("http://docs.oasis-open.org/ns/cmis/restatom/200901",
						"CollectionType"), cType.value());
		// act.setCollectionType(cType);
		act.setHref(href);
		act.setTitle(getAtomText(title, "text"));

		switch (cType) {
		case CHECKEDOUT:
			act.getAccept().add(MIMETYPE_ATOMENTRY);
			break;
		case QUERY:
			act.getAccept().add(MIMETYPE_QUERY);
			break;
		case ROOTCHILDREN:
			break;
		case ROOTDESCENDANTS:
			break;
		case TYPESCHILDREN:
			break;
		case TYPESDESCENDANTS:
			break;
		case UNFILED:
			act.getAccept().add(MIMETYPE_ATOMENTRY);
		}

		return act;
	}

	/**
	 * @author abrown
	 * 
	 * @param stmt
	 * @param pageSize
	 * @param skipCount
	 * @param retAllowable
	 * @param retAllVersions
	 * @return
	 */
	private static CmisQueryType getQueryType(String stmt, int pageSize,
			int skipCount, boolean retAllowable, boolean retAllVersions) {
		CmisQueryType cqt = new CmisQueryType();

		cqt.setStatement(stmt);
		cqt.setMaxItems(BigInteger.valueOf(pageSize));
		cqt.setIncludeAllowableActions(retAllowable);
		cqt.setSearchAllVersions(retAllVersions);
		cqt.setSkipCount(BigInteger.valueOf(skipCount));

		return cqt;
	}

	/**
	 * @author abrown
	 * 
	 * @param string
	 * @param string2
	 * @param object
	 * @param object2
	 * @return
	 */
	private static Link getAtomLink(String rel, String href, String type,
			String length, String id, String title) {
		Link l = new Link();
		l.setHref(href);
		l.setRel(rel);
		l.setType(type);
		l.setLength(length);
		l.setTitle(title);

		// add in CMIS specific info (if applicable)
		if (id != null)
			l
					.getOtherAttributes()
					.put(
							new QName(
									"http://docs.oasis-open.org/ns/cmis/restatom/200901",
									"id"), id);
		return l;
	}

	/**
	 * @author abrown
	 * 
	 * @param string
	 * @param string2
	 * @param string3
	 * @return
	 */
	private static AtomPersonConstruct getAtomPerson(String name, String email,
			String uri) {

		AtomPersonConstruct apc = new AtomPersonConstruct();
		apc.setName(name);
		apc.setEmail(email);
		apc.setUri(uri);

		return (apc);
	}

	public static CmisAllowableActionsType getAllowableActions(
			EnumBaseObjectType ot) {
		CmisAllowableActionsType aa = new CmisAllowableActionsType();

		aa.setCanDelete(true);
		aa.setCanAddToFolder(true);
		aa.setCanAddPolicy(true);
		aa.setCanGetAppliedPolicies(true);
		aa.setCanGetProperties(true);
		aa.setCanGetRelationships(true);
		aa.setCanMove(true);
		aa.setCanRemoveFromFolder(true);
		aa.setCanRemovePolicy(true);
		aa.setCanUpdateProperties(true);

		switch (ot) {
		case DOCUMENT:
			aa.setCanCancelCheckout(true);
			aa.setCanCheckin(true);
			aa.setCanCheckout(true);
			aa.setCanCreateDocument(true);
			aa.setCanDeleteContent(true);
			aa.setCanDeleteVersion(true);
			aa.setCanSetContent(true);
			aa.setCanViewContent(true);
			aa.setCanGetParents(true);
			aa.setCanGetAllVersions(true);
			break;
		case FOLDER:
			aa.setCanCreateFolder(true);
			aa.setCanGetChildren(true);
			aa.setCanGetDescendants(true);
			aa.setCanDeleteTree(true);
			aa.setCanGetFolderParent(true);
			break;
		case POLICY:
			aa.setCanCreatePolicy(true);
			break;
		case RELATIONSHIP:
			aa.setCanCreateRelationship(true);
			break;
		}

		return aa;
	}

	public <T> void writeOutput(String file, JAXBElement<T> element) {
		FileOutputStream out = null;

		try {
			out = new FileOutputStream("schema/Ex-" + file + ".xml", false);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}

		if (marshaller == null) {
			getMarshaller();
		}
		try {
			marshaller.marshal(element, out);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
	}

	private void getMarshaller() {
		try {
			jaxbContext = JAXBContext.newInstance(JAXB_PACKAGE_LIST);
			marshaller = jaxbContext.createMarshaller();

			try {
				marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
				marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
						Boolean.TRUE);

				marshaller.setProperty(
						"com.sun.xml.bind.namespacePrefixMapper",
						new NamespacePrefixMapperImpl());

			} catch (PropertyException e1) {
				System.out.println("Property Exception : " + e1.toString());
				e1.printStackTrace();
			}

		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
