/********************************************************************/
/* IBM Confidential                                                 */
/*                                                                  */
/* OCO Source Materials                                             */
/*                                                                  */
/* 5724-E76 and 5655-R17                                            */
/*                                                                  */
/* (C) Copyright IBM Corp. 2008                                     */
/*                                                                  */
/* The source code for this program is not published or otherwise   */
/* divested of its trade secrets, irrespective of what has been     */
/* deposited with the U.S. Copyright Office.                        */
/********************************************************************/

package cmis.junit.examples;

public class NamespacePrefixMapperImpl extends com.sun.xml.bind.marshaller.NamespacePrefixMapper {
	private static final String _CORE = "org.oasis_open.docs.ns.cmis.core._20090128";
	private static final String _MSG = "org.oasis_open.docs.ns.cmis.messaging._20090128";
	private static final String _ATOM = "org.w3._2005.atom";
	private static final String _APP = "org.w3._2007.app";

	/**
	 * Returns a preferred prefix for the given namespace URI.
	 * 
	 * This method is intended to be overrided by a derived class.
	 * 
	 * @param namespaceUri
	 *            The namespace URI for which the prefix needs to be found.
	 *            Never be null. "" is used to denote the default namespace.
	 * @param suggestion
	 *            When the content tree has a suggestion for the prefix to the
	 *            given namespaceUri, that suggestion is passed as a parameter.
	 *            Typicall this value comes from the QName.getPrefix to show the
	 *            preference of the content tree. This parameter may be null,
	 *            and this parameter may represent an already occupied prefix.
	 * @param requirePrefix
	 *            If this method is expected to return non-empty prefix. When
	 *            this flag is true, it means that the given namespace URI
	 *            cannot be set as the default namespace.
	 * 
	 * @return null if there's no prefered prefix for the namespace URI. In this
	 *         case, the system will generate a prefix for you.
	 * 
	 *         Otherwise the system will try to use the returned prefix, but
	 *         generally there's no guarantee if the prefix will be actually
	 *         used or not.
	 * 
	 *         return "" to map this namespace URI to the default namespace.
	 *         Again, there's no guarantee that this preference will be honored.
	 * 
	 *         If this method returns "" when requirePrefix=true, the return
	 *         value will be ignored and the system will generate one.
	 */
	public String getPreferredPrefix(String namespaceUri, String suggestion,
			boolean requirePrefix) {

		if ("http://www.w3.org/2001/XMLSchema-instance".equals(namespaceUri)) {
			return "xsi";
		}
		
		if (namespaceUri.equals(_APP)) {
			return ("app");
		}
		
		if (namespaceUri.equals(_ATOM)) {
			return ("atom");
		}
		if (namespaceUri.equals(_CORE)) {
			return ("cmis");
		}
		if (namespaceUri.equals(_MSG)) {
			return ("cmism");
		}
		
		// use default
		return suggestion;
	}
}
